/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState } from "react";
import moment from 'moment';
//imports from local files
import TableData from "../../../Components/Components/Table";
import SnackBar from "../../../../Components/SnackbarComponent";
import { Url, DeleteFilter } from "../../../Filters";
import AlertDialog from "../../../Components/Components/AlertDialog";
import LoadingSpinner from '../../../../Components/LoadingSpinner';
//imports from material ui
import { withStyles } from "@material-ui/core/styles";
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import MenuItem from "@material-ui/core/MenuItem";
import TableRow from "@material-ui/core/TableRow";
import TextField from "@material-ui/core/TextField";
import IconButton from "@material-ui/core/IconButton";
import { Button, Typography, Paper, Grid } from "@material-ui/core";
import CircularProgress from "@material-ui/core/CircularProgress";
import AddRoundedIcon from "@material-ui/icons/AddRounded";
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded";
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import DeleteIcon from "@material-ui/icons/Delete";
import { deleteRequest, getRequest } from '../../../../Components/HelperFunctions/getAxiosConfig';
//routers
import { Link } from 'react-router-dom';

let currPage = 0;

const StyledTableCell = withStyles((theme) => ({
    head: {
        backgroundColor: theme.palette.common.black,
        color: theme.palette.common.white,
    },
    body: {
        fontSize: 14,
    },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
    root: {
        "&:nth-of-type(odd)": {
            backgroundColor: theme.palette.action.hover,
        },
    },
}))(TableRow);

const Collections = () => {
    const ADD_ONE_LENGHT = [""];
    const TOP_LABELS = [
        "NOME",
        "DESCRIÇÃO",
        "DONO(A)",
        "CRIAÇÃO",
        "ATUALIZAÇÃO",
        "PRIVACIDADE",
        "VISUALIZAR",
        "DELETAR"
    ]; //Labels from Table

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [items, setItems] = useState([]); //Necessary to consult the API, data

    const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false); //controlls the state of loadind more data
    const [isUpdating, setIsUpdating] = useState(false); //controlls the state of updating data

    const [showFilter, setShowFilter] = useState(false);
    const [search, setSearch] = useState('');

    const [openAlertDialog, setOpenAlertDialog] = useState(false);
    const [deleteItem, setDeleteItem] = useState({}); //Delete Item
    const [isLoadingToDelete, setIsLoadingToDelete] = useState(null);

    const [option, setOption] = useState("Todos os usuários"); //labels of the text field 'to'

    const [snackInfo, setSnackInfo] = useState({
        message: "",
        icon: "",
        open: false,
        color: "",
    });

    const options = [
        {
            value: "private",
            label: "Privado",
        },
        {
            value: "public",
            label: "Público",
        },
    ];

    //handle snack info
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color,
        });
    };

    //handle load more items
    const LoadMoreItens = async (api) => {
        setIsLoadingMoreItems(true);
        getRequest(api,
            (data, header) => {
                const arrData = [...data];
                if (arrData.length === 0) {
                    HandleSnack(
                        "Não há mais dados para serem carregados",
                        true,
                        "warning",
                        "#FFC125"
                    );
                } else {
                    const arrItems = [...items];
                    arrItems.pop(); //Deleting the last position, that was used to display the button of load more items
                    const arrResult = arrItems.concat(arrData);
                    setItems(arrResult.concat(ADD_ONE_LENGHT));
                }
                setIsLoadingMoreItems(false);
            },
            (error) => {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                setIsLoadingMoreItems(false);
            }
        )
    };

    //Defines which row must show the circular progress
    const HandleStateCircularProgress = (i) => {
        setIsLoadingToDelete(i);
    };

    // handle update list data
    const UpdateHandler = async (api) => {
        setIsUpdating(true);
        getRequest(api,
            (data, header) => {
                HandleSnack(
                    "A lista de dados foi atualizada",
                    true,
                    "success",
                    "#228B22"
                );
                const arrData = [...data];
                setItems(arrData.concat(ADD_ONE_LENGHT));
                setIsUpdating(false);
            },
            (error) => {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                setIsUpdating(false);
            }
        )
    };

    //Called when user want to delete one institution
    async function DeleteHandler() {
        const id = deleteItem.id;
        HandleStateAlertDialog(null);
        deleteRequest(DeleteFilter("collections", id),
            (data) => {
                HandleSnack(
                    "A Coleção foi deletada com sucesso",
                    true,
                    "success",
                    "#228B22"
                );
                currPage = 0;
                HandleStateCircularProgress(null);
                UpdateHandler(Url("collections", "", `${currPage}`, "DESC"));
            },
            (error) => {
                HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                HandleStateCircularProgress(null);
            }
        )
    }

    //Controlls the state of the Alert Dialog
    const HandleStateAlertDialog = (i) => {
        const obj = { ...items[i] };
        setDeleteItem(obj);
        setOpenAlertDialog(!openAlertDialog);
    };


    // handle change of privacy
    const handleChange = (e) => {
        const value = e.target.value;
        console.log(value);
        currPage = 0;
        setIsLoaded(false)
        setOption(value);
        getRequest(Url("collections", `"privacy" : "${value}"`, `${currPage}`, "DESC"),
            (data, header) => {
                const arrData = [...data];
                setItems(arrData.concat(ADD_ONE_LENGHT));
                HandleSnack("Filtro aplicado com sucesso", true, "success", "#228B22");
                setIsLoaded(true);
            },
            (error) => {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                setIsLoaded(true);
            }
        )
    };

    //Handle the search filter
    const HandleSearch = (event) => {
        setSearch(event.target.value)
        getRequest(
            Url("collections", `"name" : "${search}"`, `${currPage}`, "DESC"),
            (data, header) => {
                const arrData = [...data];
                setItems(arrData.concat(ADD_ONE_LENGHT));
                setIsLoaded(true);
            },
            (error) => {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                setIsLoaded(true);
            }
        )
    }

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData)
            .format("LLL")
            .toString();
    };

    //getting data from server
    useEffect(() => {
        getRequest(
            Url("collections", "", `${currPage}`, "DESC"),
            (data, header) => {
                const arrData = [...data];
                setItems(arrData.concat(ADD_ONE_LENGHT));
                setIsLoaded(true);
                setError(false);
            },
            (error) => {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                setIsLoaded(false);
                setError(true);
            }
        )
    }, []);

    if (error) {
        return <div>Error: {error.message}</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {

        return <>
            <SnackBar
                severity={snackInfo.icon}
                text={snackInfo.message}
                snackbarOpen={snackInfo.open}
                color={snackInfo.color}
                handleClose={() =>
                    setSnackInfo({
                        message: "",
                        icon: "",
                        open: false,
                        color: "",
                    })
                }
            />

            <Paper style={{ padding: "1em" }}>
                <Grid container spacing={3} direction="row" alignItems="center">
                    <Grid item xs={6}>
                        <Typography variant="h4">Coleções</Typography>
                    </Grid>
                    <Grid
                        item
                        xs={6}
                    >
                        <Grid container justify="flex-end" spacing={3}>
                            <Grid item>
                                <Button
                                    variant="contained"
                                    color="secondary"
                                    disabled={isUpdating}
                                    onClick={() => {
                                        currPage = 0;
                                        UpdateHandler(
                                            Url("collections", "", `${currPage}`, "DESC")
                                        );
                                    }}
                                    startIcon={<UpdateRoundedIcon />}
                                >
                                    {isUpdating ? <CircularProgress size={24} /> : "Atualizar"}
                                </Button>
                            </Grid>
                            <Grid item>
                                <Button
                                    variant="contained"
                                    color="secondary"
                                    onClick={() => {
                                        currPage = 0;
                                        UpdateHandler(
                                            Url("collections", "", `${currPage}`, "DESC")
                                        );
                                        setShowFilter(!showFilter);
                                    }}
                                    startIcon={<FilterListRoundedIcon />}
                                >
                                    Filtrar
                                </Button>
                            </Grid>
                        </Grid>
                    </Grid>
                </Grid>

                {showFilter ? (
                    <>
                        <div style={{ height: "1em" }}></div>

                        <Grid container alignItems="center" alignContent="center" xs={12} direction="row" justify="space-between">
                            <Grid item>
                                <TextField
                                    select
                                    label="Filtro"
                                    value={option ? option : ""}
                                    onChange={handleChange}
                                    helperText="Por favor, selecione uma das opções"
                                >
                                    {options.map((option, index) => (
                                        <MenuItem
                                            key={index}
                                            value={option.value}
                                        >
                                            {option.label}
                                        </MenuItem>
                                    ))}
                                </TextField>
                            </Grid>

                            <Grid item>
                                <TextField
                                    label="Pesquisa"
                                    onChange={(event) => HandleSearch(event)}
                                >

                                </TextField>
                            </Grid>
                        </Grid>
                    </>
                ) : null}
            </Paper>

            <div style={{ height: "2em" }}></div>

            <TableData top={TOP_LABELS}>
                <TableBody>
                    {items.map((row, index) =>
                        index === items.length - 1 ? (
                            <StyledTableRow key={index}>
                                <StyledTableCell>
                                    <Button
                                        key={index}
                                        color="primary"
                                        variant="text"
                                        // disabled={isLoadingMoreItems}
                                        startIcon={<AddRoundedIcon />}
                                        disabled={isLoadingMoreItems}
                                        onClick={() => {
                                            currPage++;
                                            if (showFilter) {
                                                if (search) {
                                                    LoadMoreItens(
                                                        Url("collections", `"name" : "${search}"`, `${currPage}`, "DESC")
                                                    );
                                                }
                                                else {
                                                    LoadMoreItens(
                                                        Url("collections", `"privacy" : "${option}"`, `${currPage}`, "DESC")
                                                    );
                                                }
                                            } else {
                                                LoadMoreItens(
                                                    Url("collections", "", `${currPage}`, "DESC")
                                                );
                                            }
                                        }}
                                    >
                                        {isLoadingMoreItems ? (
                                            <CircularProgress size={24} />
                                        ) : (
                                                "Carregar mais itens"
                                            )}
                                    </Button>
                                </StyledTableCell>
                            </StyledTableRow>
                        ) : (
                                <StyledTableRow key={index}>
                                    <StyledTableCell component="th" scope="row">
                                        {row.name}
                                    </StyledTableCell>
                                    <StyledTableCell align="right">
                                        <div dangerouslySetInnerHTML={{ __html: row.description }}>
                                        </div>
                                    </StyledTableCell>
                                    <StyledTableCell align="right">
                                        {
                                            row.owner === null ? '' : row.owner.name
                                        }
                                    </StyledTableCell>
                                    <StyledTableCell align="right">
                                        {DisplayDate(row.created_at)}
                                    </StyledTableCell>
                                    <StyledTableCell align="right">
                                        {DisplayDate(row.updated_at)}
                                    </StyledTableCell>
                                    <StyledTableCell align="right">
                                        {row.privacy}
                                    </StyledTableCell>
                                    <StyledTableCell align="right">
                                        <Link to={`/admin/Collection/${row.id}`}>
                                            <IconButton onClick={() => {currPage = 0}}>
                                                <VisibilityIcon style={{ fill: "#00bcd4" }} />
                                            </IconButton>
                                        </Link>
                                    </StyledTableCell>
                                    <StyledTableCell align="right">
                                        {isLoadingToDelete === index ? (
                                            <CircularProgress size={24} color="primary" />
                                        ) : (
                                                <IconButton
                                                    onClick={() => {
                                                        HandleStateAlertDialog(index);
                                                        HandleStateCircularProgress(index);
                                                    }}
                                                >
                                                    <DeleteIcon style={{ fill: "#FF0000" }} />
                                                </IconButton>
                                            )}
                                    </StyledTableCell>
                                </StyledTableRow>
                            )
                    )}
                </TableBody>
            </TableData>

            {/* This alert will be displayed if the user click to delete an institution */}
            <AlertDialog
                open={openAlertDialog}
                OnDelete={DeleteHandler}
                deleteItem={deleteItem}
                HandleClose={() => {
                    setOpenAlertDialog(false);
                    HandleStateCircularProgress(null);
                }}
            />
        </>
    }
};
export default Collections;
