/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from 'react'
import LoadingSpinner from '../../LoadingSpinner.js'
import ContainerRedeVazia from './ContainerRedeVazia.js'
import PanelTemplateRede from '../PanelComponents/TemplateRede.js'
import { fetchAllRequest,getRequest } from '../../HelperFunctions/getAxiosConfig'

export default function TabPanelRede(props) {
    const [loading, handleLoading] = useState(true)

    const [followingList, setFollowing] = useState([])
    const [currFollowingLimit, setCurrFollowingLimit] = useState(4)
    const [loadingMoreFollowing, setLoadingFollowing] = useState(false)
    const [endOfFollowing, setEndOfFollowing] = useState(false)

    const [followersList, setFollowers] = useState([])
    const [currFollowerLimit, setFollowersLimit] = useState(4)
    const [loadingMoreFollowers, setLoadingMoreFollowers] = useState(false)
    const [endOfFollowers, setEndOfFollowers] = useState(false)

    const showMoreFollowing = (limite) => {
        setLoadingFollowing(true);
        const limit = limite;
        setCurrFollowingLimit(currFollowingLimit + limit)
        const url = `/users/${props.id}/following/User?offset=${currFollowingLimit}&limit=${limit}`;
        getRequest(url,
            (data) => {
                if (data.length >= 1) {
                    let currData = [...followingList];
                    currData = [...currData.concat(data)];
                    setLoadingFollowing(false);
                    setFollowing(currData);
                }
                else {
                    setLoadingFollowing(false);
                    setEndOfFollowing(true)
                }
            },
            (error) => { console.log(error) }
        )
    }

    const showMoreFollowers = (limite) => {
        setLoadingMoreFollowers(true);
        const limit = limite;
        setFollowersLimit(currFollowerLimit + limit)
        const url = `/users/${props.id}/followers?offset=${currFollowerLimit}&limit=${limit}`;
        getRequest(url,
            (data) => {
                if (data.length >= 1) {
                    let currData = [...followersList];
                    currData = [...currData.concat(data)];
                    setLoadingMoreFollowers(false);
                    setFollowers(currData);
                }
                else {
                    setLoadingMoreFollowers(false);
                    setEndOfFollowers(true)
                }
            },
            (error) => { console.log(error) }
        )
    }

    function handleSuccess(responseArr) {
        setFollowing(responseArr[0])
        setFollowers(responseArr[1])
        handleLoading(false)
    }

    useEffect(() => {
        handleLoading(true)
        const urls = [
            `/users/${props.id}/following/User`,
            `/users/${props.id}/followers`
        ]

        fetchAllRequest(urls, handleSuccess, (error) => { console.log(error) })
    }, [])

    return (
        <>
            {
                loading ?
                    (
                        [
                            <LoadingSpinner text={'CARREGANDO...'} />
                        ]
                    )
                    :
                    (
                        [
                            <>
                                {
                                    followingList.length === 0 && followersList.length === 0 ?
                                        (
                                            [
                                                <>
                                                    <ContainerRedeVazia />
                                                </>
                                            ]
                                        )
                                        :
                                        (
                                            <React.Fragment>
                                                <PanelTemplateRede
                                                    title={followersList.length === 1 ? "Seguidor" : "Seguidores"}
                                                    length={followersList.length}
                                                    sliceArr={followersList}
                                                    showMore={showMoreFollowers}
                                                    follower={true}
                                                    loadingMore={loadingMoreFollowers}
                                                    end={endOfFollowers}
                                                    noContentText={'Você não possui nenhum seguidor'}
                                                />

                                                <PanelTemplateRede
                                                    title={"Seguindo"}
                                                    length={followingList.length}
                                                    sliceArr={followingList}
                                                    showMore={showMoreFollowing}
                                                    follower={false}
                                                    loadingMore={loadingMoreFollowing}
                                                    end={endOfFollowing}
                                                    noContentText={'Você ainda não segue nenhum usuário'}
                                                />
                                            </React.Fragment>

                                        )
                                }
                            </>
                        ]
                    )
            }
        </>
    )
}
